------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   MASTER CONTROL BOARD
-- DEVICE ID    :   MCB SEQUENCER FPGA U22
-- PARENT MODULE:   TOP LEVEL  
-- MODULE       :   CLOCK ENABLE REGISTER
-- VERSION      :   4.0 
------------------------------------------------------------------------
-- FILE NAME    :   CLOCKENABLEREGISTER.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   05/06/2004
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 5.2
------------------------------------------------------------------------
-- DESCRIPTION  :  PROVIDES CLOCK DRIVERS FOR THE INDIVIDUAL BOARD CLOCKS. 
--
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- Version 3.4 .. Revision 0 hardware and 8 slot schroff backplane only.
-- CURRENT 8 SLOT BACKPLANE HAS BOARD CLOCKS 2 AND 3, 4 AND 5, BUSSED TOGETHER
-- THIS IMPLEMENTATION HAS SPECIFIC CODING FOR THIS SCHEME. SETTING BIT 1 ON
-- WILL RESULT IN BOTH SLOT 2 AND 3 HAVING CLOCKS.
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE	  AUTHOR  VERSION  MODIFICATIONS
-- 02/26/2003 PETER MOORE  0.1  ORIGINAL VERSION
-- 03/26/2004 PETER MOORE  3.4  CHANGED TO NON-TRISTATE DRIVERS AND INVERTED CLOCK
--                              TO COMPENSATE FOR PROPOGATION DELAY ON PCI BACKPLANE
-- 05/06/2004 PETER MOORE  4.0  ASSIGNMENTS CHANGED FOR 6 SLOT BACKPLANE WITH REV A HARDWARE.
-- 
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- CLK         : SYSTEM CLOCK
-- RST         : SYSTEM RESET
-- WRT         : WRITE STROBE FOR CLK_EN_DATA
-- CLK_EN_DATA : LOWEST 7 BITS CONTROL THE ENABLES
-- CLK_DATA_OUT: REFLECTION OF CLOCK REGISTER CONTENTS
-- CLK2        : BOARD 2 CLOCK DRIVER OUTPUT
-- CLK3        : BOARD 3 CLOCK DRIVER OUTPUT
-- CLK4        : BOARD 4 CLOCK DRIVER OUTPUT
-- CLK5        : BOARD 5 CLOCK DRIVER OUTPUT
-- CLK6        : BOARD 6 CLOCK DRIVER OUTPUT
-- CLK7        : BOARD 7 CLOCK DRIVER OUTPUT
-- CLK8        : BOARD 8 CLOCK DRIVER OUTPUT
------------------------------------------------------------------------
library IEEE;
LIBRARY UNISIM;
use IEEE.STD_LOGIC_1164.ALL;

entity ClockEnableRegister is
    Port ( CLK         : in  std_logic;
           RST         : in  std_logic;
           WRT         : in  std_logic;
           CLK_EN_DATA : in  std_logic_vector(15 downto 0);
           CLK_DATA_OUT: out std_logic_vector(15 downto 0);
           CLK2        : out std_logic;
           CLK3        : out std_logic;
           CLK4        : out std_logic;
           CLK5        : out std_logic;
           CLK6        : out std_logic;
           CLK7        : out std_logic;
           CLK8        : out std_logic
         );
end ClockEnableRegister;

architecture Behavioral of ClockEnableRegister is

    signal clk_reg : std_logic_vector(15 downto 0);
    signal clk_n : std_logic;
    signal en_clk2, en_clk3, en_clk4, en_clk5, en_clk6, en_clk7, en_clk8 : std_logic;
    signal clk2or3, clk4or5 : std_logic;

begin

------------------------------------------------------------------------
-- Update the clock enable register through reset or write strobe
------------------------------------------------------------------------
    clk_wr : process (CLK)
    begin
        if (rising_edge(CLK)) then
            if (RST = '1') then
                clk_reg <= (others => '0');
            elsif (WRT = '1') then
                clk_reg <= CLK_EN_DATA;
            end if;
        end if;
    end process;

------------------------------------------------------------------------
-- Let the rest of the world see the enables data (as it was written)
------------------------------------------------------------------------
    CLK_DATA_OUT <= clk_reg;
    clk_n <= not CLK;

------------------------------------------------------------------------
-- Decode the enables depending on 8 slot or 6 slot backplane
------------------------------------------------------------------------
    en_clk2 <= clk_reg(1) when (clk_reg(15) = '0') else clk_reg(9);
    en_clk3 <= clk_reg(2) when (clk_reg(15) = '0') else clk_reg(8);
    en_clk4 <= clk_reg(3) when (clk_reg(15) = '0') else (clk_reg(1) or clk_reg(2));
    en_clk5 <= clk_reg(4) when (clk_reg(15) = '0') else clk_reg(5);
    en_clk6 <= clk_reg(5) when (clk_reg(15) = '0') else clk_reg(6);
    en_clk7 <= clk_reg(6) when (clk_reg(15) = '0') else (clk_reg(3) or clk_reg(4));
    en_clk8 <= clk_reg(7) when (clk_reg(15) = '0') else clk_reg(7);


    CLK2 <= clk_n and en_clk2;
    CLK3 <= clk_n and en_clk3;
    CLK4 <= clk_n and en_clk4;
    CLK5 <= clk_n and en_clk5;
    CLK6 <= clk_n and en_clk6;
    CLK7 <= clk_n and en_clk7;
    CLK8 <= clk_n and en_clk8;

end Behavioral;
