------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   MASTER CONTROL BOARD
-- DEVICE ID    :   U22
-- PARENT MODULE:   SEQUENCER TOP LEVEL (SEQ_FPGA)  
-- MODULE       :   COMMAND REGISTER
-- VERSION      :   4.3 
------------------------------------------------------------------------
-- FILE NAME    :   COMMAND_REG.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   09/08/2004
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 6.2
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- The module is a 16 bit register with write enable 
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- BIT 0 	: ALWAYS TRUE
-- BIT 1	: EXTERNAL HARDWARE SYNC BIT
-- BIT 2	: TERMINAL COUNT FLAG FROM INTEGRATION TIMER
-- BIT 3	: START EXPOSURE FLAG; SET BY START EXPOSURE COMMAND; RESET BY EFR BIT(0) AND CONTROL REG BIT(0) = 0.
-- BIT 11:4 : START EXPOSURE VECTOR AS WRITTEN BY DHE COMMAND START EXP; RESET BY CONTROL REG BIT(0) = 0 (I.E SEQUENCER NOT RUNNING).
-- BIT 15:12: SOFT BITS WRITABLE FROM LOW ORDER BITS IN WRITE TO COMMAND REG.
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE	      AUTHOR  VERSION  MODIFICATIONS
-- 03/11/2003  KC       1.0    The least significant bit is tied to logic '1' to implement unconditional jump.
-- 07/08/2003	KC		   2.0	 Start Exposure Flag, START EXPOSURE VECTOR bits,TC and SYNC bits are Added
-- 05/13/2004  PCM      4.2    CANCELLATION OF START BIT + VECTOR WHEN SEQUENCER NOT RUNNING
-- 09/08/2004  PCM      4.3    ADJUSTED BIT POSITIONS FOR PAN WRITE FROM LEAST SIG. NIBBLE TO MS NIBBLE
-- 10/04/2005  PCM      4.3    MADE THE CMD BIT REGISTER RESETS ASYNCHRONOUS TO ALLOW TESTING ON NEXT SEQ INSTRUCTION
--
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- Write 		: Write strobe from DhePanDecode logic.
-- SysClk 		: System clock
-- STNC 		: Input from DHE SYNC signal
-- TC 			: Terminal count bit from integration timer counter
-- StartExp 	: Start exposure received flag, validates the staart exposure vector in bits 11 downto 4.
-- StartExpRst	: Reset strobe from EFR register to cancel the start exposure bit
-- DataIn 		: Data bus in for start exp and cmd reg writes;		
-- Reset 		: Good o'l reset
-- CmdBits 		: Register contents out.
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity Command_Reg_V465 is
    Port ( 	Write 		: in  std_logic;
		   	SysClk 		: in  std_logic;
            STNC 	    : in  std_logic; -- bit 1
           	TC 			: in  std_logic; -- bit 2
		   	SE_IN    	: in  std_logic; -- bit 3
            SE_OUT      : out std_logic;
            SE_RDY      : inout  std_logic;
            MASTER      : in  std_logic;
            StartExpRst	: in  std_logic;
            PAUSE       : in  std_logic;
            DataIn 		: in  std_logic_vector(15 downto 0);		
		   	Reset 		: in  std_logic;
            SeqRunning  : in  std_logic;
           	CmdBits 	: out std_logic_vector(15 downto 0)
         );
end Command_Reg_V465;

architecture Behavioral of Command_Reg_V465 is
    signal SE_Latch : std_logic;
begin

	CmdBits(0) <= '1';
	CmdBits(1) <= STNC when (SE_Latch = '1') else PAUSE;
	CmdBits(2) <= TC;
	CmdBits(3) <= SE_Latch when (MASTER = '0') else (SE_Latch and SE_RDY);
    SE_OUT <= SE_Latch;
    SE_RDY <= SE_Latch when (SE_Latch = '0' and MASTER = '0') else 'Z';
       
	StartExpBits : process (Reset, SeqRunning, StartExpRst, Sysclk)
	begin
        if (Reset='1' or SeqRunning = '0') then
           CmdBits(11 downto 4) <=  (others => '0');
           SE_Latch <= '0';
        elsif (StartExpRst = '1') then 					-- Reset the start exposure flag from enables register
            SE_Latch  <= '0';
		elsif rising_edge (SYSCLK) then
            if (SE_IN = '1') then
               CmdBits(11 downto 4) <= DataIn(7 downto 0);	-- Load in the start vector
               SE_Latch  <= '1';							-- Set the start exposure flag true
            end if;
		end if;
	end process;

	PanWriteBits : process (Reset,Sysclk)
	begin
		if rising_edge (SYSCLK) then
            if (Reset = '1') then     --RESET active High
               CmdBits(15 downto 12)  <= (others => '0');
            elsif (Write = '1') then  --Write active High
               CmdBits(15 downto 12)  <= DataIn(15 downto 12);
            end if;
		end if;
	end process;

end Behavioral;

