------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   MASTER CONTROL BOARD
-- DEVICE ID    :   MCB SEQUENCER FPGA U22
-- PARENT MODULE:   TOP LEVEL  
-- MODULE       :   PAN COMMAND DECODER
-- VERSION      :   4.66 
------------------------------------------------------------------------
-- FILE NAME    :   PAN_COMMAND_DECODE.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   04/15/2008
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 7.2
------------------------------------------------------------------------
-- DESCRIPTION  :  RECEIVES 32 BIT WORDS FROM THE FPGA INTERFACE, DECODES
-- THE DHE COMMAND AND ASSEMBLES THE DATA STRUCTURE FOR TRANSMISSION ON 
-- THE SEQUENCER BUS
--
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- WRITES TO ADDRESS 0xFFFF (NOMINAL BOARD CODE REVISION IDENT) ARE CONSIDERED
-- TO BE A HARD RESET REQUEST. IN SIMILAR FASHION, WRITES TO 0xFFFE (BOARD FUNCTION
-- IDENTITY) ARE DECODED TO PROVIDE A SOFT RESET TO THE ADDRESSED BOARD.
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE	  AUTHOR  VERSION  MODIFICATIONS
-- 02/26/2003  PCM   2.1  ORIGINAL VERSION
-- 05/07/2003  KAVI  2.1  MODIFIED TO INCLUDE LOCAL BOARD OPERATIONS
-- 07/08/2003  PCM   2.2  ADDED RESET DECODING LOGIC
-- 04/22/2004  pcm   4.0  ADAPTED FOR HARDWARE REVISION A.
-- 04/15/2008  pcm   4.66 Modified echo enable to fix problem when writing 32-bit words
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- CLK          : SYSTEM CLOCK
-- RST          : SYSTEM RESET
-- DATA_IN_VALID: FLAG FROM FPGA INTERFACE TO SIGNAL DATA AVAILABLE
-- DATA_IN_RD   : READ STROBE BACK TO THE FPGA INTERFACE.
-- DATA_IN      : 32 BIT RAW DATA FROM THE FPGA INTERFACE
-- MODE         : DECODED MODE WORD FROM DHE DATA
-- DEVADDR      : DECODED DEV ADDRESS FROM DHE DATA
-- ADDRDATA     : DECODED DATA / ADDRESS FROM DHE DATA
-- SEQ_BUS_REQ  : THIS MODULES REQUEST FOR ACCESS TO TEH SEQUENCER BUS 
-- SEQ_BUS_ACK  : SEQUENCER BUS MULTIPLEXOR BUS GRANT SIGNAL
-- BOARD_SELECT : SEPERATE BOARD SELECT SIGNALS FROM DHE DATA
-- PIX_FIFO_WRT: WRITE CODE TO THE FIBER FPGA PIXEL BUS FIFO PORT
-- START_EXP    : START EXPOSURE STROBE TO SEQUENCER MODULE
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity Pan_Command_Decode_V466 is
    Port ( CLK          : in  std_logic;
           RESET        : in  std_logic;
           DATA_IN_VALID: in  std_logic;
           DATA_IN_RD   : inout std_logic;
           DATA_IN      : in  std_logic_vector(31 downto 0);
           ECHO_DATA    : out std_logic_vector(31 downto 0);
           MODE         : inout std_logic_vector(1 downto 0);
           DEVADDR      : inout std_logic_vector(4 downto 0);
           ADDRDATA     : inout std_logic_vector(31 downto 0);
           LOCDATA      : out std_logic_vector(31 downto 0);
           LOCADDR      : out std_logic_vector(15 downto 0);
           LOCSEL       : out std_logic;
           SEQ_BUS_REQ  : out std_logic;
           SEQ_BUS_ACK  : in  std_logic;
           BOARD_SELECT : inout std_logic_vector(7 downto 0);
           PIXFIFO_WRT_N: out std_logic;
           START_EXP    : out std_logic;
           ECHO_REQ     : out std_logic;
           WRT          : out std_logic;
           RD           : out std_logic;
           WRT_PROG     : out std_logic;
           SYNC_FLAG    : out std_logic;
           ASYNC_PENDING: in  std_logic;
           HARD_RST     : out std_logic;
           SOFT_RST     : out std_logic
         );
end Pan_Command_Decode_V466;

architecture Behavioral of Pan_Command_Decode_V466 is

------------------------------------------------------------------------
-- CONSTANTS FOR PAN COMMAND CODES IN FIRST WORD (BITS 31 -> 30)
------------------------------------------------------------------------
    constant DHE_EXP  : std_logic_vector(1 downto 0) := "00";
    constant DHE_READ : std_logic_vector(1 downto 0) := "01";
    constant DHE_WRT  : std_logic_vector(1 downto 0) := "10";
    constant DHE_ASYN : std_logic_vector(1 downto 0) := "11";

------------------------------------------------------------------------
-- CONSTANTS FOR SEQUENCER BUS MODE WORD
------------------------------------------------------------------------
    constant SEQ_RST  : std_logic_vector(1 downto 0) := "00";
    constant SEQ_RD   : std_logic_vector(1 downto 0) := "01";
    constant SEQ_WR16 : std_logic_vector(1 downto 0) := "10";
    constant SEQ_WR32 : std_logic_vector(1 downto 0) := "11";

------------------------------------------------------------------------
-- CONSTANTS FOR RESET MODES
------------------------------------------------------------------------
    constant HARD_RST_ADDR : std_logic_vector(15 downto 0) := x"FFFF";  -- BOARD CODE REVISION IDENTITY REGISTER
    constant SOFT_RST_ADDR : std_logic_vector(15 downto 0) := x"FFFE";  -- BOARD FUNCTIONAL IDENTITY REGISTER
 
------------------------------------------------------------------------
-- LOCAL VARIABLES
------------------------------------------------------------------------
    constant RD_ECHO_DELAY : std_logic_vector(3 downto 0) := "1000";    -- Number of clocks to delay for a read.
    type statetype is (idle, exp_1, asyn_1, rd_1, rd_2, RdEchoWait, wr_1, wr_2, wr_3, fin);
    signal State : statetype;
    signal dhe_echo   :std_logic;
    signal dhe_cmd    : std_logic_vector(1 downto 0);
    signal dhe_addr   : std_logic_vector(15 downto 0);
    signal dhe_board  : std_logic_vector(7 downto 0);
    signal addr_buffr : std_logic_vector(15 downto 0);
    signal dhe_board_latch : std_logic_vector(7 downto 0);
    signal wrt_type   : std_logic_vector(1 downto 0);
    signal RdEchoCntr : std_logic_vector(3 downto 0);
    signal wrt_32, wrt_32_flag ,locwrt : std_logic;
    signal soft_rst_req: std_logic;
    signal hard_rst_req: std_logic;
    signal EchoEnable  : std_logic;
	signal START_EXP_s : std_logic;
    signal AsynRcvd    : std_logic;
begin

------------------------------------------------------------------------
-- Static assignments
------------------------------------------------------------------------
    dhe_cmd   <= DATA_IN(31 downto 30); -- Convenience only !
    dhe_echo  <= not DATA_IN(29);
    dhe_board <= DATA_IN(23 downto 16);
    dhe_addr  <= DATA_IN (15 downto 0);
    ECHO_REQ <= EchoEnable and DATA_IN_RD;
    wrt_type(1) <= dhe_addr(15) or dhe_addr(14) or dhe_addr(13) or dhe_addr(12) or dhe_addr(11);
    wrt_type(0) <= dhe_addr(10) or dhe_addr(9) or dhe_addr(8) or dhe_addr(7) or dhe_addr(6) or dhe_addr(5);
    wrt_32 <= not (wrt_type(0) or wrt_type(1));
    hard_rst_req <= '1' when (dhe_addr = HARD_RST_ADDR and dhe_cmd = DHE_WRT) else '0';
    soft_rst_req <= '1' when (dhe_addr = SOFT_RST_ADDR and dhe_cmd = DHE_WRT) else '0';
    AsynRcvd <= DATA_IN(31) and DATA_IN(30);

------------------------------------------------------------------------
-- Decoder state machine
------------------------------------------------------------------------

    cmd_decode : process (CLK)
    begin
        if (rising_edge(CLK)) then
            if (RESET = '1') then
                State <= idle;
                SEQ_BUS_REQ <= '0';
                BOARD_SELECT <= (others => '0');
                PIXFIFO_WRT_N <= '1';
                DATA_IN_RD <= '0';
                START_EXP_s <= '0';
            else
                case State is
                    when idle =>
                        BOARD_SELECT <= (others => '0');    -- Default condition in Idle
                        if ((DATA_IN_VALID and (not DATA_IN_RD)) = '1') then
                            if (ASYNC_PENDING = '1' and AsynRcvd = '0') then
                                ECHO_DATA <= x"C00100FF";
                                DATA_IN_RD <= '1';
                                EchoEnable <= '1';                  -- Always echo an async response  
                                State <= asyn_1;
                            else
                                ECHO_DATA <= DATA_IN;
                                EchoEnable <= dhe_echo;             -- define whether we echo or not on this possible command dhe_echo = true means no echovrequired
                                DATA_IN_RD <= '1';
                                dhe_board_latch <= dhe_board;
                                case dhe_cmd is
                                    when DHE_EXP =>         -- Start exposure command from DHE
        			                    START_EXP_s <= '1';      -- Set the strobe to true.
                                        ADDRDATA <= x"0000" & dhe_addr;
                                        EchoEnable <= '0';
                                        State <= exp_1;          -- used to set sequencer mpu flag
                                    when DHE_READ =>             -- Read command from DHE
                                        MODE <= SEQ_RD;          -- Correct mode for a read opperation
                                        DEVADDR <= (others => '0');
                                        ADDRDATA <= dhe_addr(15 downto 0) & x"0000"; 
                                        if (dhe_echo = '1') then           -- Do we need to delay for the echo ??
                                            RdEchoCntr <= (others => '0'); -- Reset the clock counter
                                            State <= RdEchoWait;
                                        else    
                                            BOARD_SELECT <= dhe_board;
                                            SEQ_BUS_REQ <= '1';      -- Assert the bus request flag
                                            State <= rd_1;           -- Setup the next state
                                        end if;
                                    when DHE_WRT =>        -- Write command from DHE, wait for the data rdy strobe to go false
                                        wrt_32_flag <= wrt_32;        -- 1 bit memory to retain mode for next state
                                        if (hard_rst_req = '1') then
                                            MODE <= SEQ_RST;
                                            DEVADDR <= (others => '1');  -- Device address contains reset code for soft / hard function
                                        elsif (soft_rst_req = '1') then
                                            MODE <= SEQ_RST;
                                            DEVADDR <= (others => '0');
                                        elsif (wrt_32 = '1') then
                                            MODE <= SEQ_WR32;          -- Correct mode for a 32 bit write
                                            DEVADDR <= dhe_addr(4 downto 0);
                                        else
                                            MODE <= SEQ_WR16;          -- Correct mode for a 16 bit write
                                            addr_buffr <= dhe_addr;    -- Catch the write address for later
                                            DEVADDR <= (others => '0');
                                        end if;
                                        State <= wr_1;
                                    when DHE_ASYN =>       -- Async message acknowledge;
                                        SYNC_FLAG <= '1';
                                        State <= asyn_1;
                                    when others =>      -- Errr !
                                        State <= idle;
                                end case;
                            end if;
    					else
    						DATA_IN_RD <= '0';
                            State <= Idle;
                        end if;
                    when asyn_1 =>    -- dummy state to cancel the read flag
                        DATA_IN_RD <= '0';      -- Cancel the read strobe 
                        SYNC_FLAG <= '0';       --Cancel the strobe (that should clear async_pending !).
                        State <= fin;
                    when exp_1 =>    -- Start exposure command here
                        DATA_IN_RD <= '0';      -- Cancel the read strobe 
                         State <= fin;
                    when RdEchoWait =>  -- Wait here for the echo to be transfered and sent via the pixel fpga first.
                        DATA_IN_RD <= '0';      -- Cancel the read strobe and wait for the bus acknowledge. 
                        if (RdEchoCntr < RD_ECHO_DELAY) then
                            RdEchoCntr <= RdEchoCntr + 1;
                            State <= RdEchoWait;
                        else
                            BOARD_SELECT <= dhe_board_latch;
                            SEQ_BUS_REQ <= '1';      -- Assert the bus request flag
                            State <= rd_1;           -- Setup the next state
                        end if;    
                    when rd_1 =>
                        DATA_IN_RD <= '0';      -- Cancel the read strobe and wait for the bus acknowledge. 
                        if ((SEQ_BUS_ACK = '1') or (BOARD_SELECT(0) = '1')) then
                            State <= rd_2;
                        else
                            State <= rd_1;
                        end if;
                    when rd_2 =>    -- One clock cycle later, pulse the write fifo control lines
                        PIXFIFO_WRT_N <= '0';         -- Signal the pixel bus fifo on the fiber fpga to pick up the data. 
                        State <= fin;                 -- Go back to idle state.
                    when wr_1 =>
                        DATA_IN_RD <= '0';      -- Cancel the read strobe for the write command
                        State <= wr_2;          -- Wait state for data ready flag to clear.
                    when wr_2 =>
                        if (DATA_IN_VALID = '1') then
                            if (wrt_32_flag = '1') then
                                ADDRDATA <= DATA_IN;
                            else
                                ADDRDATA <= addr_buffr(15 downto 0) & DATA_IN(15 downto 0);
                            end if;
                            ECHO_DATA <= DATA_IN;   -- Set up the echo data
                            BOARD_SELECT <= dhe_board_latch;
                            SEQ_BUS_REQ <= '1';
                            DATA_IN_RD <= '1';      -- Acknowledge the second word
                            locwrt <= '1';
                            State <= wr_3;
                        else
                            State <= wr_2;
                        end if;
                    when wr_3 =>
                        DATA_IN_RD <= '0';      -- Cancel the read strobe
                        if ((SEQ_BUS_ACK = '1') or (BOARD_SELECT(0) = '1')) then
                            SEQ_BUS_REQ <= '0';
                            BOARD_SELECT <= (others => '0');    -- Kill the board select line(s)
                            State <= fin;                       -- Go back to idle state.
                            locwrt <= '0';
                        else
                            State <= wr_3;
                        end if;
                    when fin =>
                        locwrt <= '0';
                        SEQ_BUS_REQ <= '0';
                        BOARD_SELECT <= (others => '0');
                        PIXFIFO_WRT_N <= '1';
                        START_EXP_s <= '0';
                        State <= idle;
                    when others =>
                        DATA_IN_RD <= '0';      -- Cancel the read strobe for the write command
                        State <= idle;
                end case;
            end if;
		end if;
    end process;



   decode : process (CLK)
    begin
        if (rising_edge(CLK)) then
            LOCSEL <= BOARD_SELECT(0);
            WRT_PROG <= '0';
            SOFT_RST <= '0';
            WRT <= '0';
            RD <= '0';
			START_EXP <= '0';
            if (BOARD_SELECT(0) = '1') then
                case MODE is
                    when SEQ_RD =>
                        LOCADDR <= ADDRDATA(31 downto 16) ;
                        WRT <= '0';
                        RD <= '1';
                    when SEQ_WR16 =>
                        LOCDATA <= x"0000" & ADDRDATA(15 downto 0) ;
                        LOCADDR <= ADDRDATA(31 downto 16) ;
                        if (ADDRDATA(31 downto 26) = "010000") then
                            WRT_PROG <= locwrt;
                        end if;
                        WRT <= locwrt;
                    when SEQ_WR32 =>
                        LOCDATA <= ADDRDATA;
                        LOCADDR <= "00000000000" & DEVADDR;
                        WRT <= locwrt;
                    when SEQ_RST =>
                        HARD_RST <= DEVADDR(4) and DEVADDR(3) and DEVADDR(2) and DEVADDR(1) and DEVADDR(0);  -- Note that hard reset is latched forever !!
                        SOFT_RST <= not (DEVADDR(4) or DEVADDR(3) or DEVADDR(2) or DEVADDR(1) or DEVADDR(0));
                    when others =>
                        LOCDATA <= ADDRDATA;
                        WRT <= '0';
                end case;
            elsif (START_EXP_s = '1') then
			    LOCDATA <= ADDRDATA;
				START_EXP <= '1';
	        end if;
        end if;
    end process;    

end Behavioral;
