------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   MASTER CONTROL BOARD
-- DEVICE ID    :   MCB SEQUENCER FPGA U22
-- PARENT MODULE:   TOP LEVEL  
-- MODULE       :   POWER SUPPLY SYNC GENERATOR
-- VERSION      :   4.64 
------------------------------------------------------------------------
-- FILE NAME    :   POWERSYNCGEN.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   03/28/2007
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 7.2
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- PROVIDES A SEQUENCE OF NEGATIVE GOING SYNCHRONIZATION PULSES TO SYNC THE
-- SWITCH MODE POWER SUPPLY TO THE PIXEL CONVERSION RATE.
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- SYNC_IN ALWAYS GENERATES A SYNC_OUT PULSE.
-- THE FREQUENCY OF THE SYNC_IN PULSES IS MEASURED; IF THIS FREQUENCY IS LESS
-- THAN 500KHz THEN THE THE SYNC_OUT FREQUENCY IS MULTIPLIED BY A VALUE SUFFICIENT 
-- TO MAINTAIN THE POWER SUPPLY SWITCHING FREQUENCY BETWEEN 500KHz AND 1MHz.
-- 
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE	  AUTHOR  VERSION  MODIFICATIONS
-- 03/28/2007  PCM   4.64   INITIAL BUILD
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- RESET       : RESET THIS LOGIC BLOCK
-- SYSCLK      : DATA VALID ON RISING EDGE
-- SYNC_IN     : SEQUENCER GENERATED PULSE TO DRIVE SYNC LOGIC
-- SYNC_OUT    : DRIVER TO THE POWER SUPPLY FOR SWITCHING CONTROL
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity PowerSyncGen_V464 is
    Port ( 
	 			SYSCLK 	: in std_logic;
           	RESET 	: in std_logic;
           	SYNC_IN 	: in std_logic;
           	SYNC_OUT : out std_logic;
				LOCKED   : out std_logic
			);
end PowerSyncGen_V464;

architecture Behavioral of PowerSyncGen_V464 is

	constant PulseInPeriod  : std_logic_vector(5 downto 0) := "100111";  -- Retrigger delay for sync in pulse = 1us
	                                                                     -- Sets upper limit of SYNC_OUT to 1MHz
	constant PulseOutPeriod : std_logic_vector(4 downto 0) := "10011";   -- Sync_out pulse width = 500ns
	constant KHZ_500_Limit  : std_logic_vector(11 downto 0) := x"050";
	constant KHZ_250_Limit  : std_logic_vector(11 downto 0) := x"0A0";
	constant KHZ_125_Limit  : std_logic_vector(11 downto 0) := x"140";
	constant KHZ_6P25_Limit  : std_logic_vector(11 downto 0) := x"280";

	signal PulseInCntr  : std_logic_vector(5 downto 0); 
	signal PulseOutCntr : std_logic_vector(4 downto 0);
	signal MonoTrigger  : std_logic;
	signal SyncDetect   : std_logic;
	type SyncFreqType is (stage_1, stage_2, stage_3, stage_4, stage_5);
	signal SyncFreqState: SyncFreqType;
	signal SyncFreqCntr_0, SyncFreqCntr_1  : std_logic_vector(11 downto 0);	  -- allows strobe frequency measurements between 40KHz & 800KHz.
	signal SyncFreqLatch : std_logic_vector(6 downto 0);		-- Needs dynamic range between 40 and 80 for 500KHz => 1MHz.
	signal SyncFreqValid : std_logic;
	signal FreqGenCntr   : std_logic_vector(6 downto 0);
	signal FreqGenStrobe : std_logic;

begin

--------------------------------------------------
--- This process detects the SYNC_IN pulse and ---
--- produces SyncDetec for 1 SYSCLK period     ---
--- The SyncDetect is non-retriggerable for the --
--- period of PulseInCntr x SYSCLK             ---
--------------------------------------------------
	SyncInMonoProcess : process (RESET, SYSCLK)
		begin
			if (RESET = '1') then
				PulseInCntr <= (others => '0');
				SyncDetect <= '0';
			elsif (rising_edge (SYSCLK)) then
				if (PulseInCntr /= "00000") then
					SyncDetect <= '0';
					PulseInCntr <= PulseInCntr - 1;
				elsif (SYNC_IN = '1') then
					SyncDetect <= '1';
					PulseInCntr <= PulseInPeriod;
				else
					SyncDetect <= '0';
				end if;
			end if;
		end process;

------------------------------------------------------
--- A simple monostable pulse generator to produce ---
--- an output pulse of PulseOutCntr x SYSCLK width ---
--- It is triggered for every SYNC_IN event and from -
--- the frequency multiplier output to maintain    ---
--- the sync_out frequency between 500 & 1000 KHz  ---
------------------------------------------------------ 
	MonoTrigger <= SyncDetect or FreqGenStrobe;

	SyncOutMonoProcess : process (RESET, SYSCLK)
		begin
			if (RESET = '1') then
				SYNC_OUT <= '1';
				PulseOutCntr <= (others => '0');
			elsif (rising_edge (SYSCLK)) then
				if (PulseOutCntr /= "00000") then
					PulseOutCntr <= PulseOutCntr - 1;
				elsif (MonoTrigger = '1') then
					PulseOutCntr <= PulseOutPeriod;
					SYNC_OUT <= '0';
				else
					SYNC_OUT <= '1';
				end if;
			end if;
		end process;


------------------------------------------------------
--- Measures the period between SYNC_IN events and ---
--- applies a simple majority of 2 filter to the   ---
--- measurement. If the SYNC_IN period is stable   ---
--- the process defines a period for the internal  ---
--- frequency generator and enables it. This is to ---
--- maintain the SYNC_OUT frequency between the    ---
--- operational limits of the power supply (i.e. 500 -
--- to 1000 KHz                                    ---
------------------------------------------------------

	SyncFreqCntrProcess : process (RESET, SYSCLK)
	begin
		if (RESET = '1') then
			SyncFreqValid <= '0';	
			LOCKED <= '0';
		elsif (rising_edge (SYSCLK)) then
			case SyncFreqState is
				when stage_1 =>		-- Wait for the first sync pulse to come through
					if (SyncDetect = '1') then
						SyncFreqCntr_0 <= (others => '0');
						SyncFreqState <= stage_2;
					else
						SyncFreqState <= stage_1;
					end if;	
				when stage_2 =>		-- Wait for second sync pulse to come through
					if (SyncDetect = '1') then
						SyncFreqCntr_1 <= (others => '0');
						SyncFreqState <= stage_3;
					elsif (SyncFreqCntr_0(11) = '1') then
						LOCKED <= '0';
						SyncFreqState <= stage_1;		-- If the timer has overflowed, give up
					else
						SyncFreqCntr_0 <= SyncFreqCntr_0 + 1;
						SyncFreqState <= stage_2;
					end if;	
				when stage_3 =>		-- Wait for third sync pulse to arrive
					if (SyncDetect = '1') then
						SyncFreqState <= stage_4;
					elsif (SyncFreqCntr_1(11) = '1') then
						LOCKED <= '0';
						SyncFreqState <= stage_1;		-- If the timer has overflowed, give up
					else
						SyncFreqCntr_1 <= SyncFreqCntr_1 + 1;
						SyncFreqState <= stage_3;
					end if;	
				when stage_4 =>		-- Compare the two measurements to see that they are the same
					if (SyncFreqCntr_0 = SyncFreqCntr_1) then
						LOCKED <= '1';
						SyncFreqState <= stage_5;
					else
						LOCKED <= '0';
						SyncFreqState <= stage_1;		-- Give up if the periods are different i.e. avoid jitter.
					end if;
				when stage_5 =>		-- Compute a reasonable power supply switching frequency
					if (SyncFreqCntr_0 < KHZ_500_Limit) then
						SyncFreqLatch <= SyncFreqCntr_0(6 downto 0);		-- Let the pixel rate drive the supply directly
						SyncFreqValid <= '1';											-- Enable the generator so it can just flywheel
					elsif (SyncFreqCntr_0 < KHZ_250_Limit) then
						SyncFreqLatch <= SyncFreqCntr_0(7 downto 1);		-- Multiply the pixel rate x 2
						SyncFreqValid <= '1';											-- Enable the generator
					elsif (SyncFreqCntr_0 < KHZ_125_Limit) then
						SyncFreqLatch <= SyncFreqCntr_0(8 downto 2);		-- Multiply the pixel rate x 4
						SyncFreqValid <= '1';											-- Enable the generator
               elsif (SyncFreqCntr_0 < KHZ_6P25_Limit) then
						SyncFreqLatch <= SyncFreqCntr_0(9 downto 3);		-- Multiply the pixel rate x 8
						SyncFreqValid <= '1';											-- Enable the generator
					else
						SyncFreqLatch <= SyncFreqCntr_0(10 downto 4);  -- Multiply the pixel rate x 16
						SyncFreqValid <= '1';											-- Enable the generator						
					end if;
					SyncFreqState <= stage_1;		-- Start another measurement period.
			end case;
		end if;
	end process;

--------------------------------------------------------
--- Simply generates afrequency that it is commanded ---
--- by the SyncFreqCntr process. The output pulses are -
--- OR'ed with the SYNC_IN events to produce a       ---
--- frequency multiplication (if required)           ---
-------------------------------------------------------- 

	FreqGenProcess : process (RESET, SYSCLK, SyncDetect)
	begin
		if (RESET = '1' or SyncDetect = '1') then
			FreqGenStrobe <= '0';
			FreqGenCntr <= (others => '0');
		elsif (rising_edge(SYSCLK) and SyncFreqValid = '1') then
			if (FreqGenCntr >= SyncFreqLatch) then
				FreqGenStrobe <= '1';
				FreqGenCntr <= (others => '0');
			else
				FreqGenStrobe <= '0';
				FreqGenCntr <= FreqGenCntr + 1;
			end if; 		
		end if;
	end process;

end Behavioral;
