------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   GENERIC
-- DEVICE ID    :   U22 (SEQUENCER FPGA)
-- PARENT MODULE:   SEQUENCER TOP LEVEL (SEQ_FPGA)  
-- MODULE       :   ENABLES REGISTER
-- VERSION      :   4.64 
------------------------------------------------------------------------
-- FILE NAME    :   ENABLES_REG.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   03/28/2007
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 7.2
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- THIS REGISTER CONTAINS SOME STROBES AND CONTROL BITS USED IN CONTROL 
-- CONTEXT FOR CERTAIN SEQ_MPU FUNCTIONS. THE WRITE TO THIS REGISTER IS
-- FROM THE SEQ_MPU ITSELF WHEN THE BOARD SELECT VALUE = 1 (I.E. MCB).
-- DATA IS LOADED FROM THE CURRENT PATTERN POINTER AND PROCESSED ACCORDING
-- TO ITS' BIT POSITION AS OUTLINED BELOW.
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- PATTERN MEMORY BITS AND THEIR FUNCTION
-- BIT								BIT
-- 31								15 FORCE SYNC OUT HIGH
-- 30								14 FORCE SYNC OUT LOW
-- 29								13 PULSE POWER SUPPLY SYNC LOGIC
-- 28								12
-- 27								11
-- 26								10
-- 25								 9 RESET LSR AUTO CANCEL MODE
-- 24								 8 SET LSR AUTO CANCEL MODE
-- 23								 7
-- 22								 6
-- 21								 5
-- 20								 4
-- 19								 3 
-- 18								 2 STOP INTEGRATION TIMER
-- 17								 1 START INTEGRATION TIMER
-- 16								 0 RESET START EXP FLAG IN CMD REG
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE	        AUTHOR  VERSION  MODIFICATIONS
-- 07/10/2003   PCM     0.1      INITIAL VERSION
-- 02/15/2005   PCM     4.4      ADDED FRAME START PULSE OUTPUT
-- 10/28/2005   PCM     4.5      REMOVED FRAME START PULSE WHICH IS NOW IN EVENTS REGISTER
-- 03/28/2007	 PCM     4.64		ADDED POWER SUPPLY SYNC BIT
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- CLK	 		: SEQUENCER CLOCK
-- WRT_ENABLES 	: STROBE TRUE WHEN BOARD SELECT = 1 AND MODE = WRT32
-- PATTERN_DATA : PATTERN DATA INPUT
-- ENABLE_INT 	: ENABLE LINE TO INTEGRATION TIMER, STARTS AND STOPS INTEGRATION TIMER
-- RST_STARTBIT : STROBE TO RESET THE START EXPOSURE FLAG IN CMD REGISTER 
-- SYNC_OUT 	: STROBE TO GO TO SYNC LINE ON OTHER DHEs'
-- FRAME_START  : STROBE LINE TO ANNOUNCE BEGINING OF READOUT FRAME. DEBUGGING PURPOSE.
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity enables_reg_V464 is
    Port ( CLK	 		: in  std_logic;
           SEQ_RUNNING  : in  std_logic;
           WRT_ENABLES 	: in  std_logic;
           PATTERN_DATA : in  std_logic_vector(31 downto 0);
           ENABLE_INT 	: out std_logic;
           AUTO_CANCEL  : out std_logic;
           RST_STARTBIT : out std_logic;
           SYNC_OUT 	   : out std_logic;
			  PWR_SYNCBIT  : out std_logic
         );
end enables_reg_V464;

architecture Behavioral of enables_reg_V464 is

begin

------------------------------------------------------------------------
-- RESET THE START EXPOSURE FLAG IN THE CMD REGISTER WHEN BIT 0 HIGH
------------------------------------------------------------------------
	start_exp : process (CLK)
	begin
		if (rising_edge(CLK)) then
			if (WRT_ENABLES = '1') then
				RST_STARTBIT <= PATTERN_DATA(0);
			else
				RST_STARTBIT <= '0';
			end if;
		end if;
	end process;

------------------------------------------------------------------------
-- ENABLE THE INTEGRATION TIMER COUNTER WHEN BIT 1 HIGH
-- DISABLE THE INTEGRATION TIMER COUNTER WHEN BIT 2 HIGH - HAS PRIORITY
------------------------------------------------------------------------
	int_rs : process (CLK)
	begin
		if (rising_edge(CLK)) then
            if (SEQ_RUNNING = '1') then
    			if (WRT_ENABLES = '1' and PATTERN_DATA(2) = '1') then
    				ENABLE_INT <= '0';
    			elsif (WRT_ENABLES = '1' and PATTERN_DATA(1) = '1') then
    				ENABLE_INT <= '1';
    			end if;
            else
                ENABLE_INT <= '0';
            end if;    
		end if;
	end process;

------------------------------------------------------------------------
-- ENABLE THE BOARD SELECT AUTO CANCEL FEATURE WHEN BIT 8 HIGH
-- DISABLE THE BOARD SELECT AUTO CANCEL FEATURE WHEN BIT 9 HIGH - HAS PRIORITY
------------------------------------------------------------------------
	auto_rs : process (CLK)
	begin
		if (rising_edge(CLK)) then
            if (SEQ_RUNNING = '1') then
    			if (WRT_ENABLES = '1' and PATTERN_DATA(9) = '1') then
    				AUTO_CANCEL <= '0';
    			elsif (WRT_ENABLES = '1' and PATTERN_DATA(8) = '1') then
    				AUTO_CANCEL <= '1';
                end if;
            else
    		    AUTO_CANCEL <= '1';
            end if;
		end if;
	end process;


------------------------------------------------------------------------
-- PULSE THE POWER SUPPLY SYNC FLAG WHEN BIT 13 HIGH
------------------------------------------------------------------------
	pwr_sync_process : process (CLK)
	begin
		if (rising_edge(CLK)) then
			if (WRT_ENABLES = '1') then
				PWR_SYNCBIT <= PATTERN_DATA(13);
			else
				PWR_SYNCBIT <= '0';
			end if;
		end if;
	end process;

------------------------------------------------------------------------
-- SET THE SYNC_OUT FLAG  HIGH WHEN BIT 15 HIGH
-- SET THE SYNC_OUT FLAG  LOW WHEN BIT 14 HIGH
------------------------------------------------------------------------
	sync_rs : process (CLK)
	begin
		if (rising_edge(CLK)) then
            if (SEQ_RUNNING = '1') then
    			if (WRT_ENABLES = '1' and PATTERN_DATA(14) = '1') then
    				SYNC_OUT <= '0';
    			elsif (WRT_ENABLES = '1' and PATTERN_DATA(15) = '1') then
    				SYNC_OUT <= '1';
    			end if;
            else
    		    SYNC_OUT <= '0';
            end if;
		end if;
	end process;

end Behavioral;
