------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   MASTER CONTROL BOARD
-- DEVICE ID    :   MCB SEQUENCER FPGA U22
-- PARENT MODULE:   TOP LEVEL  
-- MODULE       :   FPDP INTERFACE - PAN => DHE
-- VERSION      :   4.0 
------------------------------------------------------------------------
-- FILE NAME    :   FPDP_TM.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   04/22/2004
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 5.2
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- PROVIDES THE DATA AND CONTROL INTERFACE BETWEEN THE FIBER FPGA AND THE
-- SYSTRAN FPDP RECEIVER PORT I.E. PAN => DHE. IMPLEMENTS A 64 WORD BUFFER 
-- FOR INCOMING DATA. DECODES THE PIO_1 AND PIO_2 SIGNALS TO GATE THE WRITES
-- TO THE FIFO (BOTH LOW) AND GENERATE A RESET OUT SIGNAL (BOTH HIGH).
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- RESET_OUT pulse duration is 1 clock cycle.
-- PIO1 and PIO2 debounce delay is 1us.
-- RESET_OUT retrigger delay is 50ms.
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE	  AUTHOR  VERSION  MODIFICATIONS
-- 02/13/2003  PCM   2.1   MODIFIED RESET LOGIC TO PREVENT FALSE TRIGGERING
-- 07/04/2003  PCM   2.2   MODIFIED RESET TO ACCOMODATE SHORTER PIO WIDTHS (5ms)
-- 04/22/2004  PCM   4.0   ADAPTED TO REVISION A HARDWARE ARCHITECTURE. MOVED
--                         MODULE FROM FIBER FPGA TO SEQ FPGA.
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- TSTROBE     : FROM J4 PIN 63 OF SYSTRAN BOARD. CLK OUTPUT FROM FPDP TM PORT
-- TDVALID_N   : FROM J4 PIN 55 OF SYSTRAN BOARD. LOW TRUE DATA VALID ON DATA_IN
-- TDIR_N      : FROM J4 PIN 53 OF SYSTRAN BOARD. GENERAL PURPOSE INPUT FROM FPDP / DHE
-- TSUSPEND_N  : TO J4 PIN 57 OF SYSTRAN BOARD. FLOW CONTROL TO FPDP.
-- TNRDY_N     : TO J4 PIN 61 OF SYSTRAN BOARD. ASSERT WHEN NOT READY FOR DATA. 
-- TERROR      : FROM J4 PIN 45 OF SYSTRAN BOARD. ERROR ON FPDP BOARD.
-- TSYNC_N     : FROM J4 PIN 51 OF SYSTRAN BOARD. FRAME SYNC FOR THIS TDVALID_N STROBE.
-- PIO1_OUT    : FROM J4 PIN 58 OF SYSTRAN BOARD. GENERAL PURPOSE OUTPUT FROM DHE
-- PIO2_OUT    : FROM J4 PIN 50 OF SYSTRAN BOARD. GENERAL PURPOSE OUTPUT FROM DHE
-- DATA_IN     : 32 BIT DATA FROM SYSTRAN BOARD.
-- DATA_OUT    : 32 BIT DATA WORD OUTPUT TO FPGA LOGIC
-- SYNC        : SYNC SIGNAL SYNCRONIZED WITH FIFO.
-- ASYNC_FLAG  : SET AFTER RESET WHILE AWAITING AN ASYNC CMD FOR SYNCRONIZATION.
-- DATA_RDY    : THERE IS VALID DATA WAITING IN THE FIFO
-- RST         : RESET THIS LOGIC BLOCK
-- CLK         : DATA VALID ON RISING EDGE
-- RD          : READ ENABLE FOR DATA_OUT ON RISING CLOCK EDGE
-- ERR         : STATUS BIT THAT LATCHES THE TERROR FLAG, CLEARED ONLY BY RST.
-- RST_OUT     : TDIR_N SYNCRONIZED OUTPUT.
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity FPDP_TM is
    Port ( 
            TSTROBE     : in  std_logic;
            TDVALID_N   : in  std_logic;
            TDIR_N      : in  std_logic;
            TSUSPEND_N  : out std_logic;
            TNRDY_N     : out std_logic;
            TERROR      : in  std_logic;
            TSYNC_N     : in  std_logic;
            PIO1_OUT    : in  std_logic;
            PIO2_OUT    : in  std_logic;
            DATA_IN     : in  std_logic_vector(31 downto 0);
            DATA_OUT    : out std_logic_vector(31 downto 0);
            SYNC        : out std_logic;
            DIR         : out std_logic;
            DATA_RDY    : out std_logic;
            RESET_IN    : in  std_logic;
            SYSCLK      : in  std_logic;
            RD          : in  std_logic;
            ERR         : out std_logic;
            RESET_OUT   : out std_logic
          );
end FPDP_TM;

architecture Behavioral of FPDP_TM is

------------------------------------------------------------------------
-- Set up a RAM block for the fifo itself
-- Infer a Distributed RAM register block 33 bits wide x 64 deep
------------------------------------------------------------------------
    type ram_type is array (63 downto 0) of std_logic_vector (31 downto 0); 
    signal fifo_ram : ram_type;                                               -- 32 bit data fifo
    signal fifo_wrt_addr, fifo_rd_addr : std_logic_vector(5 downto 0);        -- Address pointers / counters for fifo access
    signal error : std_logic;
    signal fifo_empty, fifo_wrt_enable : std_logic;
    signal tdir_flag, tdir_out, sync_flag, sync_out : std_logic;
    signal local_dat : std_logic_vector(31 downto 0);                         -- Local buffer for fifo data
    signal reset_active      : std_logic;                                       -- reset derived from pio_1 & 2 in high state and sync'ed to CLK.
    signal retrigger_timeout : std_logic;                                       -- Flag active if we're timing out a retrigger holdoff.
    signal rst_cntr          : std_logic_vector(5 downto 0);
    constant rst_delay       : std_logic_vector(5 downto 0)  := "101000";       -- reset debounce delay set to 1.0 us.
    constant retrigger_dly   : std_logic_vector(11 downto 0) := "010110000110"; -- Used to hold off a retrigger event on reset signal for 50ms.
    signal retrigger_cntr    : std_logic_vector(11 downto 0);
    signal retrigger_subcntr : std_logic_vector(11 downto 0);

begin

------------------------------------------------------------------------
-- Static assignments
------------------------------------------------------------------------
    TSUSPEND_N <= '1';                      -- Possible future use for FIFO full state
    TNRDY_N    <= '1';
    DATA_RDY   <= not fifo_empty;
    RESET_OUT  <= reset_active;

------------------------------------------------------------------------
-- FIFO Write process and address manipulation (note tstrobe used !)
------------------------------------------------------------------------    
    FiFoWrtProc : process (TSTROBE)
    begin
        if (rising_edge(TSTROBE)) then
            if (TDVALID_N = '0') then
                fifo_ram(conv_integer(fifo_wrt_addr)) <= DATA_IN;
            end if; 
        end if; 
    end process;

    FiFoAdrProc : process(TSTROBE, RESET_IN)
    begin
        if (RESET_IN = '1') then
                fifo_wrt_addr <= (others => '0');
        elsif (rising_edge(TSTROBE)) then
            if (TDVALID_N = '0') then
                fifo_wrt_addr <= fifo_wrt_addr + 1;
            end if;
        end if;
    end process;

------------------------------------------------------------------------
-- FIFO Read process and address manipulation
------------------------------------------------------------------------    
    fifo_rd : process (SYSCLK)
    begin
        if (rising_edge(SYSCLK)) then
            if (RESET_IN = '1') then
                fifo_rd_addr <= (others => '0');   
            elsif (RD = '1' and fifo_empty = '0') then
                 fifo_rd_addr <= fifo_rd_addr + 1;
            end if; 
        end if; 
    end process;
            
    DATA_OUT <= fifo_ram(conv_integer(fifo_rd_addr));

    fifo_cntrl : process (SYSCLK)
    begin
        if (falling_edge(SYSCLK)) then
            if (fifo_rd_addr = fifo_wrt_addr) then
                fifo_empty <= '1';
            else
                fifo_empty <= '0';
            end if;
        end if;
    end process;

------------------------------------------------------------------------
-- Sync signal syncronization
------------------------------------------------------------------------
    fpdp_sync : process (TSTROBE,RESET_IN) 
    begin 
        if (RESET_IN = '1') then                -- Async reset here because systran also reset and tstrobe dissapears
            sync_flag <= '0';
        elsif (rising_edge (TSTROBE)) then      -- Use TSTROBE clock as reference
            if (TSYNC_N = '0') then
                sync_flag <= '1';
            elsif (sync_out = '1') then         -- Assures a positive edge always ... Never used !!
                sync_flag <= '0';
            end if;
        end if;
    end process;
 
    sync_sync : process (SYSCLK)
    begin
        if (rising_edge(SYSCLK)) then
            if (sync_flag = '1') then
                sync_out <= '1';
            else
                sync_out <= '0';
            end if;
        end if;
    end process;
    
    SYNC <= sync_out;

------------------------------------------------------------------------
-- Tdir signal syncronization
------------------------------------------------------------------------
    fpdp_tdir : process (TSTROBE,RESET_IN) 
    begin 
        if (RESET_IN = '1') then                -- Async reset here because systran also reset and tstrobe dissapears
            tdir_flag <= '0';
        elsif (rising_edge (TSTROBE)) then      -- Use TSTROBE clock as reference
            tdir_flag <= not TDIR_N;
        end if;
    end process;

    tdir_sync : process (SYSCLK)
    begin
        if (rising_edge(SYSCLK)) then
            tdir_out <= tdir_flag;
        end if;
    end process;

    DIR <= tdir_out;

------------------------------------------------------------------------
-- Error signal latch
------------------------------------------------------------------------
    fpdp_error : process (TSTROBE,RESET_IN) 
    begin 
        if (RESET_IN = '1') then                -- Async reset here because systran also reset and tstrobe dissapears
            error <= '0';
        elsif (rising_edge (TSTROBE)) then      -- Use TSTROBE clock as reference
            if (TERROR = '1') then              -- Lock any errors that have occured till next reset
                error <= '1';
            end if;
        end if;
    end process;

    ERR <= error;

------------------------------------------------------------------------
-- reset signal synchronization
-- This process 'debounces' PIO1 and PIO2 for 1us, then sends a reset
-- strobe out and will not retrigger until 50ms later.
-- This effectively eliminates any dependence on PAN reset pulse width.
------------------------------------------------------------------------
    RstSyncProc : process (SYSCLK)
    begin
        if (rising_edge(SYSCLK)) then
            if (retrigger_timeout = '1') then
                if (retrigger_cntr < retrigger_dly) then
                    if (retrigger_subcntr = retrigger_dly) then     -- This delay counter is compound
                        retrigger_cntr <= retrigger_cntr + 1;       -- hence the delay counter loops through sub counter
                        retrigger_subcntr <= (others => '0');
                    else
                        retrigger_subcntr <= retrigger_subcntr + 1;
                    end if;
                else
                    retrigger_timeout <= '0';
                end if;
            elsif (reset_active = '1') then
                    reset_active <= '0';
                    retrigger_timeout <= '1';
                    retrigger_cntr <= (others => '0');
                    retrigger_subcntr <= (others => '0');
            elsif (PIO1_OUT = '1' and PIO2_OUT = '1') then
                if (rst_cntr = rst_delay) then
                    reset_active <= '1';         -- Delays 1 us before issuing reset .. Just to debounce anything wierd
                    rst_cntr <= (others => '0'); -- and sets up clock for retrigger delay.
                else
                    rst_cntr <= rst_cntr + 1;
                end if;
            end if;
        end if;
    end process;
    
end Behavioral;
