------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   MASTER CONTROL BOARD
-- DEVICE ID    :   MCB SEQUENCER FPGA U22
-- PARENT MODULE:   TOP LEVEL  
-- MODULE       :   SEQUENCER <=> PIXEL INTERFACE PORT
-- VERSION      :   4.0 
------------------------------------------------------------------------
-- FILE NAME    :   FPGA_INTERFACE.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   04/22/2004
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 5.2
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- PROVIDES FOR A BIDIRECTIONAL 32 BIT INTERFACE BETWEEN THE FIBER AND
-- SEQUENCER FPGA'S ON THE MCB. THE PHYSICAL DATA PATH IS ONLY 16 BITS
-- AND SO A DOUBLE FIFO IS IMPLEMENTED TO ALLOW FOR THE BOTTLENECK.
-- IN ADDITION, SUPPLIES THE SIGNALS RESET_IN AND RESET_OUT TO AND FROM THE PARTNER.
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES : GENERAL XFER SCHEME FROM INITIATOR POINT OF VIEW 
-- NOTES              1   2   3   4   5   6   7   8  
-- FPGA_REQ_OUT  -----________--------________---------
-- FPGA_ACK_IN   ---------________--------________-----
-- FPGA_TRI_OUT  -------------________________---------
-- FPGA_DATA_OUT ------------X HIGH  X  LOW  X---------
-- INITIATOR     IDLE WR1     WR2     WR3     IDLE       TX STATE DESCRIPTOR
-- PARTNER       IDLE     RD1     RD2     RD3     IDLE   RX STATE DESCRIPTOR
--    
--  1   INITIATOR REQUESTS BUS FROM PARTNER BY TAKING FPGA_REG_OUT LOW.
--  2   PARTNER GRANTS BUS REQUEST BY SIGNALLING FPGA_ACK_IN LOW.
--  3   INITIATOR ENABLES TRISTATE BUS DRIVERS WITH HIGH 16 BITS WORD ON 
--      FPGA_DATA_OUT AND SETS FPGA_REQ_OUT HIGH.
--  4   PARTNER ACCEPTS DATA AND SIGNALS XFER BY SETTING FPGA_ACK_IN HIGH.
--  5   INITAIATOR SETS LOW 16 BIT WORD AND SETS FPGA_REQ_OUT LOW.
--  6   PARTNER ACCEPTS LOW 16 BITS WORD AND SIGNALS WITH FPGA_ACK_IN LOW
--  7   INITIATOR TERMINATES XFER BY DISABLING TRISTATES AND SETTING 
--      FPGA_REQ_OUT HIGH.
--  8   PARTNER STASHES DATA TO THE RX FIFO AND RELEASES THE TRANSACTION 
--      BY SIGNALLING FPGA_ACK_IN HIGH.  
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE	       AUTHOR   VERSION  MODIFICATIONS
-- 02/24/2003 Peter Moore 2.2    Original version.
--
-- 03/04/2003 Peter Moore 2.3    Added a wait count that is active after 
-- reaching idle state from a prior read transaction before attempting to
-- initiate a write transaction. This will reduce the number of dead lock 
-- occurences when a block of data is to be transfered in one direction.
--
-- 04/22/2004 Peter Moore 4.0    Converted code to run without external
-- tristate buffers and added command channel interface. All this for the
-- new revision A MCB hardware.
--  
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- FPGA_DATA   : 16 BIT DATA INPUT FROM PARTNER
-- FPGA_DATA_OUT  : 16 BIT DATA OUTPUT TO TRISTATE BUFFERS
-- FPGA_REQ_N  : PARTNER BUS REQUEST SIGNAL IN
-- FpgaReqOut_n : OUR BUS REQUEST SIGNAL OUT
-- FPGA_ACK_N  : PARTNER BUS ACKNOWLEDGE SIGNAL IN
-- FpgaAckOut_n : OUR BUS ACKNOWLEDGE SIGNAL OUT
-- FpgaDatEnab_n  : OUR TRISTATE BUFFER ENABLE
-- FpgaReqEnab_n  : TRISTATE ENABLE LINE FOR THE REQUEST CONTROL SIGNAL
-- FpgaAckEnab_n  : TRISTATE ENABLE LINE FOR THE ACKNOWLEDGE CONTROL SIGNAL
-- FPGA_SYNC_OUT  : OUR 'EXTRA' SIGNAL FOR DATA BUS TO PARTNER
-- FPGA_SYNC   : PARTNERS EXTRA SIGNAL FOR DATA BUS IN
-- SYNC_IN   : SYNC SIGNAL USES THE 'EXTRA' SIGNAL TO XFER
-- SYNC_OUT  : SYNC SIGNAL FROM PARTNER
-- WR        : WRITE DATA TO TX FIFO STROBE
-- WR_BUSY   : CURRENTLY FIXED AT FALSE
-- DATA_IN   : DATA WRITEN TO TX FIFO WHEN WR STROBE TRUE ON RISING EDGE OF SYSCLK
-- SYSCLK       : BOARD CLOCK, RISING EDGE TRUE
-- RESET       : RESET THIS MODULE.
-- RD        : READ ONE 32 BIT WORD FROM RX FIFO WHEN TRUE
-- DATA_RDY  : SIGNALS DATA AVAILABLE ON RX FIFO STACK
-- DATA_OUT  : 32 BIT DATA OUTPUT FROM RX FIFO
------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;            

entity FPGA_INTERFACE is
    Port ( FPGA_DATA : inout std_logic_vector(15 downto 0);
           FPGA_SYNC : inout std_logic;
           FPGA_REQ_N  : inout std_logic;
           FPGA_ACK_N  : inout std_logic;
           WRITE     : in  std_logic;
           DATA_IN   : in  std_logic_vector(31 downto 0);
           SYNC_IN   : in  std_logic;
           CMD_IN    : in  std_logic;
           SYSCLK    : in  std_logic;
           RESET     : in  std_logic;
           READ      : in  std_logic;
           DATA_RDY  : out std_logic;
           DATA_OUT  : out std_logic_vector(31 downto 0);
           SYNC_OUT  : out std_logic;
           CMD_RDY   : out std_logic;
           CMD_ACK   : in  std_logic
         );

end FPGA_INTERFACE;

architecture Behavioral of FPGA_INTERFACE is

------------------------------------------------------------------------
-- State machine signales
------------------------------------------------------------------------
    type fpga_statetype is (IfcIdle, IfcWr_1, IfcWr_2, IfcWr_3, IfcRd_1, IfcRd_2, IfcRd_3, IfcFin);
    signal IfcState : fpga_statetype;
    signal retry : std_logic;                           -- Used when bus contention occurs to retry the xfer.
    signal lock_cntr : std_logic_vector(5 downto 0);    -- Used to time out bus contention
    signal wait_cntr : std_logic_vector(5 downto 0);    -- Used in slave mode to avoid deadlock occurences
    constant wait_rate : std_logic_vector(5 downto 0) := "000011"; -- Used with the wait counter during idle.
    constant lock_rate : std_logic_vector(5 downto 0) := "000111"; -- Change this so that the two partners
                                                                   -- do not have the same rate and retries 
                                                                   -- will not collide

------------------------------------------------------------------------
-- Set up a RAM block for the fifo itself
-- Infer a Distributed RAM register block 33 bits wide x 64 deep
-- We need the extra (32) bit to keep the sync signal bit syncronous
-- to the data
-- changed to 8 deep
------------------------------------------------------------------------
    type ram_type is array (7 downto 0) of std_logic_vector (33 downto 0); 
    signal tx_fifo, rx_fifo : ram_type;
 
    signal tx_buffr : std_logic_vector(33 downto 0);
    signal tx_wrt_addr, tx_rd_addr, rx_wrt_addr, rx_rd_addr : std_logic_vector(2 downto 0);
    signal tx_fifo_empty, rx_fifo_empty : std_logic;
    signal tx_fifo_rd, rx_fifo_wr : std_logic;

------------------------------------------------------------------------
-- 32 bit => 16 bit mux and 16 bit => 32 bit demux signals
------------------------------------------------------------------------
    signal tx_data_reg, rx_data_reg : std_logic_vector(33 downto 0);
    signal rx_data_out : std_logic_vector(33 downto 0);
    signal Hi_Lo_n : std_logic;
    signal reset_neg : std_logic;

------------------------------------------------------------------------
-- Tristate control signals
------------------------------------------------------------------------
    signal FpgaReqEnab_n, FpgaAckEnab_n, FpgaDatEnab_n : std_logic;
    signal FpgaReqOut_n, FpgaAckOut_n : std_logic;
begin

------------------------------------------------------------------------
-- Static assignments
------------------------------------------------------------------------
    FPGA_DATA <= (others => 'Z') when (FpgaDatEnab_n = '1') else
                 tx_buffr(31 downto 16) when (Hi_Lo_n = '1') else
                 tx_buffr(15 downto 0);

    FPGA_SYNC <= 'Z' when (FpgaDatEnab_n = '1') else
                 tx_buffr(33) when (Hi_Lo_n = '1') else 
                 tx_buffr(32);
                  
    FPGA_REQ_N  <= 'Z' when (FpgaReqEnab_n = '1') else FpgaReqOut_n;
    FPGA_ACK_N  <= 'Z' when (FpgaAckEnab_n = '1') else FpgaAckOut_n;

------------------------------------------------------------------------
-- State machine that drives the interface
------------------------------------------------------------------------
    fpga_ifc : process (SYSCLK)
    begin
        if (rising_edge(SYSCLK)) then
            if(RESET = '1') then
                IfcState <= IfcIdle;
                retry <= '0';
                tx_fifo_rd <= '0';
                rx_fifo_wr <= '0';
                FpgaReqOut_n <= '1';
                FpgaAckOut_n <= '1';
                FpgaDatEnab_n <= '1';
                FpgaReqEnab_n <= '1';         -- Disable the driver on the request line
                FpgaAckEnab_n <= '1';         -- Disable the driver on the acknowledge line
                wait_cntr <= (others => '0');
            else
                case IfcState is
                    when IfcIdle =>
                        Hi_Lo_n <= '1';                   -- Multiplex the high 16 bits from the tx_data_reg onto the output bus (but not enabled)
                        if (FPGA_REQ_N = '0') then        -- Is a request pending from our partner ?
                            FpgaAckEnab_n <= '0';         -- Enable the driver on the acknowledge line
                            FpgaAckOut_n <= '0';          -- Tell our partner that he can use the bus.
                            IfcState <= IfcRd_1;
                            wait_cntr <= (others => '0'); -- Reset the wait counter to see if there is more data coming.
                        elsif (retry = '1') then
                            FpgaReqEnab_n <= '0';         -- Enable the driver on the request line
                            FpgaReqOut_n <= '0';          -- Try to xfer our data again after a bus contention event
                            retry <= '0';                 -- Kill the flag that got us here.
                            IfcState <= IfcWr_1;
                        elsif (wait_cntr > wait_rate) then
                            if (tx_fifo_empty = '0') then  -- Do we have data to xfer to our partner ?
                                FpgaReqEnab_n <= '0';      -- Enable the driver on the request line
                                FpgaReqOut_n <= '0';       -- Set our request flag low to ask our partner for the bus.
                                tx_buffr <= tx_data_reg;   -- Grab the word off the tx fifo;
                                IfcState <= IfcWr_1;
                            else
                                IfcState <= IfcIdle;       -- Hang around till needed.
                            end if;
                        else
                            wait_cntr <= wait_cntr + 1;
                            IfcState <= IfcIdle; 
                        end if;
                    when IfcWr_1 =>  -- Wait here for partner to send ack low so we can enable the triIfcState buffers
                        if (FPGA_ACK_N = '0') then
                            tx_fifo_rd <= '1';          -- and increment the read address pointer,
                            FpgaDatEnab_n <= '0';       -- Enable the tristates when we get the ackknowledgement.
                            FpgaReqOut_n <= '1';        -- and set our request line false to say the data is there.
                            IfcState <= IfcWr_2;        -- Continue ....
                        elsif (lock_cntr = lock_rate) then
                            FpgaReqOut_n <= '0';        -- Release our request and begin again.
                            retry <= '1';               -- and set the retry flag.
                            IfcState <= IfcFin;         -- We have timed out, we are probably locked up (i.e both partners requested the bus simultainously).
                        else
                            lock_cntr <= lock_cntr + 1;
                            IfcState <= IfcWr_1;        -- Stay here and wait for acknowledge.
                        end if;
                    when IfcWr_2 => -- Wait till partner signals reception of the data high 16 bits of data.
                        tx_fifo_rd <= '0';              -- Cancel the read strobe
                        if (FPGA_ACK_N = '1') then
                            Hi_Lo_n <= '0';             -- Multiplex the low 16 bits of the tx_data_reg onto the output bus (now enabled)
                            FpgaReqOut_n <= '0';        -- and set our request line true again to say the low 16 bit data is there.                            
                            IfcState <= IfcWr_3;        -- Continue ...
                        else
                            IfcState <= IfcWr_2;        -- Otherwise just hang around.
                        end if;
                    when IfcWr_3 => -- Wait here till partner signals receipt of low 16 bits of data 
                        if (FPGA_ACK_N = '0') then
                            FpgaReqOut_n <= '1';        -- Finish the handshake process by sending REQ false
                            FpgaDatEnab_n <= '1';       -- and turn off our data bus drivers.
                            IfcState <= IfcFin;         -- Transaction closed, go clean up.
                        else
                            IfcState <= IfcWr_3;
                        end if;
                    when IfcRd_1 => -- Wait here for partner to signal that data is available by dropping req.
                        if (FPGA_REQ_N = '1') then
                            rx_data_reg(31 downto 16) <= FPGA_DATA; -- and stash the data into the high 16 bits
                            rx_data_reg(33) <= FPGA_SYNC;  -- Pick up the SYNC bit as a command flag
                            FpgaAckOut_n <= '1';        -- Set our acknowledge false to signal fiRESET xfer complete.
                            IfcState <= IfcRd_2;        -- and go wait for next word.
                        else
                            IfcState <= IfcRd_1;        -- Wait for the slow man ...
                        end if;
                    when IfcRd_2 => -- Wait till request line goes true again.
                        if (FPGA_REQ_N = '0') then
                            rx_data_reg(15 downto 0) <= FPGA_DATA;
                            rx_data_reg(32) <= FPGA_SYNC;  -- Pick up the SYNC bit - If it was true for the first xfer then this is the read / write flag.
                            FpgaAckOut_n <= '0';        -- Signal our reception of last word.
                            rx_fifo_wr <= '1';          -- and push the data onto the fifo stack.
                            IfcState <= IfcRd_3;        -- Continue ...
                        else
                            IfcState <= IfcRd_2;        -- Or stay here.
                        end if;
                    when IfcRd_3 => -- Wait for bus release and tidy up
                        rx_fifo_wr <= '0';              -- Reset the fifo write flag.
                        if (FPGA_REQ_N = '0') then
                            FpgaAckOut_n <= '1';
                            IfcState <= IfcFin;
                        else
                            IfcState <= IfcRd_3;        -- wait for slow coach ..
                        end if;
                    when IfcFin =>
                        lock_cntr <= (others => '0');   -- Clear our bus contention counter.
                        wait_cntr <= (others => '0');   -- Set up the wait counter to time out for contention
                        FpgaReqOut_n <= '1';            -- Switch off everything.
                        FpgaAckOut_n <= '1';
                        FpgaDatEnab_n <= '1';           -- Disable the data drivers
                        FpgaReqEnab_n <= '1';           -- Disable the driver on the request line
                        FpgaAckEnab_n <= '1';           -- Disable the driver on the acknowledge line
                        IfcState <= IfcIdle;            -- Go back to initial condition.
                    when others =>
                        IfcState <= IfcIdle;
                end case;
            end if;
        end if;
    end process;

------------------------------------------------------------------------
-- tx fifo (fiber => sequencer fpga) write process and address manipulation
------------------------------------------------------------------------
    tx_wrt : process (SYSCLK) 
    begin 
        if (rising_edge (SYSCLK)) then
            if (RESET = '1') then
                tx_wrt_addr <= (others => '0');
            elsif (WRITE = '1') then 
                tx_fifo(conv_integer(tx_wrt_addr)) <= (CMD_IN & SYNC_IN & DATA_IN);
                tx_wrt_addr <= tx_wrt_addr + 1;
            end if;
        end if; 
    end process;
 
------------------------------------------------------------------------
-- tx fifo (fiber => sequencer fpga) read process and address manipulation
------------------------------------------------------------------------
    
    tx_rd : process (SYSCLK)
    begin
        if (rising_edge(SYSCLK)) then
            if (RESET = '1') then
                tx_rd_addr <= (others => '0');
                reset_neg <= '1';
            elsif (tx_fifo_rd = '1') then
                tx_rd_addr <= tx_rd_addr + 1;
                reset_neg <= '0';
            end if; 
        end if; 
    end process;

    tx_data_reg <= tx_fifo(conv_integer(tx_rd_addr));

    tx_fifo_empty <= '1' when (tx_rd_addr = tx_wrt_addr) else '0';

------------------------------------------------------------------------
-- rx fifo (sequencer => fiber fpga) write process and address manipulation
------------------------------------------------------------------------
    rx_wrt : process (SYSCLK) 
    begin 
        if (rising_edge (SYSCLK)) then
            if (RESET = '1') then
                rx_wrt_addr <= (others => '0');
            elsif (rx_fifo_wr = '1') then 
                rx_fifo(conv_integer(rx_wrt_addr)) <= rx_data_reg;
                rx_wrt_addr <= rx_wrt_addr + 1;
            end if;
        end if; 
    end process;
 
------------------------------------------------------------------------
-- rx fifo (sequencer => fiber fpga) read process and address manipulation
------------------------------------------------------------------------

    rx_data_out <= rx_fifo(conv_integer(rx_rd_addr));
    
    rx_rd : process (SYSCLK)
    begin
      if (falling_edge(SYSCLK)) then 
            if (reset_neg = '1') then
                rx_rd_addr <= (others => '0');
            elsif (rx_fifo_empty = '0') then
                if (rx_data_out(33) = '1' and CMD_ACK = '1') then
                    rx_rd_addr <= rx_rd_addr + 1;  
                elsif (rx_data_out(33) = '0' and READ = '1') then 
                    rx_rd_addr <= rx_rd_addr + 1;
                end if;
            end if; 
        end if; 
    end process;

    dt_rdy : process (SYSCLK)
    begin
        if (rising_edge(SYSCLK)) then
            DATA_RDY <= not (rx_fifo_empty or rx_data_out(33));
            CMD_RDY  <= rx_data_out(33) and not rx_fifo_empty;
        end if;
    end process;
    DATA_OUT <= rx_data_out(31 downto 0);
    SYNC_OUT <= rx_data_out(32);

    rx_fifo_empty <= '1' when (rx_rd_addr = rx_wrt_addr) else '0';

end Behavioral;
